<?php

namespace Drupal\jsonapi_enhancers\Plugin\jsonapi\FieldEnhancer;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\jsonapi_extras\Plugin\ResourceFieldEnhancerBase;
use Shaper\Util\Context;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a field enhancer to convert embedded entities to absolute URLs.
 *
 * @ResourceFieldEnhancer(
 *   id = "convert_link",
 *   label = @Translation("Convert to Link"),
 *   description = @Translation("Transforms an embedded entity into an absolute URL.")
 * )
 */
class ConvertLinkEnhancer extends ResourceFieldEnhancerBase implements ContainerFactoryPluginInterface {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * Constructs a ConvertLinkEnhancer object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file URL generator service.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    LanguageManagerInterface $language_manager,
    LoggerChannelFactoryInterface $logger_factory,
    EntityTypeManagerInterface $entityTypeManager,
    FileUrlGeneratorInterface $fileUrlGenerator
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->languageManager = $language_manager;
    $this->logger = $logger_factory->get('jsonapi_enhancers');
    $this->entityTypeManager = $entityTypeManager;
    $this->fileUrlGenerator = $fileUrlGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('language_manager'),
      $container->get('logger.factory'),
      $container->get('entity_type.manager'),
      $container->get('file_url_generator')
    );
  }

  /**
   * Processes the data to convert relative or entity URIs to absolute URLs.
   *
   * @param mixed $data
   *   The data to transform.
   * @param \Shaper\Util\Context $context
   *   The transformation context.
   *
   * @return mixed
   *   The transformed data.
   */
  protected function doUndoTransform($data, Context $context) {
    if (isset($data['url'])) {
      $base_url = \Drupal::request()->getSchemeAndHttpHost();
      $data['url'] = $base_url . $data['url'];
    }
    elseif (isset($data['uri'])) {
      if (preg_match('/^entity:node\/(\d+)$/', $data['uri'], $matches)) {
        $nid = $matches[1];
        $url = Url::fromRoute('entity.node.canonical', ['node' => $nid]);
        $data['uri'] = $url->setAbsolute()->toString();
      }
    }
    return $data;
  }

  /**
   * Transforms the input value.
   *
   * @param mixed $value
   *   The value to transform.
   * @param \Shaper\Util\Context $context
   *   The transformation context.
   *
   * @return mixed
   *   The transformed value.
   */
  protected function doTransform($value, Context $context) {
    // Implement transformation logic here if needed.
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getOutputJsonSchema(): array {
    return ['type' => 'object'];
  }

}
